#! /usr/bin/env python3
u""" Modul pro Z-Wave zařízení """

import datetime
import json
import requests
import os
import statistics
from influxdb import InfluxDBClient

import config

DEBUG = os.environ.get('DEBUG', False)

cfg_heating = config.ConfigHeating()
cfg_influxdb = config.ConfigInfluxDB()

# tsdb - time series database
tsdb = InfluxDBClient(
    host=cfg_influxdb.host,
    port=cfg_influxdb.port
)
tsdb.switch_database(
    cfg_influxdb.database
)

min_from_two_values = lambda a, b : a if a < b else b
max_from_two_values = lambda a, b : a if a > b else b


class Device:
    u""" Základní společná třída pro všechna Z-Wave zařízení """
    # TODO předělat na _all_instances
    all_instances = set()

    @classmethod
    def get_set_of_instances(cls):
        return cls.all_instances

    @classmethod
    def header(cls):
        return "obyvák kuchyň kluci holky koupelna chodba topení"

    @classmethod
    def info(cls, radiator_v_obyvaku, radiator_v_kuchyni, radiator_u_kluku, radiator_u_holek, radiator_v_koupelne, radiator_v_chodbe, elektricky_kotel):
        return "{:>6s}{:>7s}{:>7s}{:>7s}{:>7s}{:>7s}{:>4s}".format(
                radiator_v_obyvaku.get_temperature_as_string(),
                radiator_v_kuchyni.get_temperature_as_string(),
                radiator_u_kluku.get_temperature_as_string(),
                radiator_u_holek.get_temperature_as_string(),
                radiator_v_koupelne.get_temperature_as_string(),
                radiator_v_chodbe.get_temperature_as_string(),
                elektricky_kotel.is_on_or_off_as_string(),
         )


    def __init__(self, hub, device_id, instance_id, _ElementID, _Name, _Description):
        self._hub = hub
        # Internal ID
        self._device_id = device_id
        # _instance_id
        self._instance_id = instance_id
        # _command_class
        # BASIC is 0x20
        self.Name = _Name
        # External ID used in URL
        self.ElementID = _ElementID
        self.Description = _Description
        self.url_info_level = hub.url_info_level.format(
            self._device_id, self._instance_id, 37
        )
        self.url_command_on = hub.url_command_on.format(
            self._device_id, self._instance_id, 37
        )
        self.url_command_off = hub.url_command_off.format(
            self._device_id, self._instance_id, 37
        )
        # ověř přes URL a nastav zda je zařízení zapnuté
        self.enabled = True if self.check_status_via_url() == "ZAP" else False
        self.temperature = None
        # using high value to allow simple lambda function for comparison
        self.temperature_min = 100
        # using low value to allow simple lambda function for comparison
        self.temperature_max = -100
        self.temperatures = tuple()
        self.temperature_median = None
        self.count_of_mismatches = 0
        Device.all_instances.add(self)

    def update_temperature_from_influxdb(self):
        u""" Načte teplotu z InfluxDB. Tam ji ukládá InfluxDb Statistics Collector na hubu """

        query = 'SELECT time, level FROM "device.%s-49-1" ORDER BY DESC LIMIT 1' % (self.ElementID)
        results = tsdb.query(
            query
        )
        _time = results.raw['series'][0]['values'][0][0]
        _temperature = results.raw['series'][0]['values'][0][1]
        timestamp = "{:%H:%M:%S}".format(datetime.datetime.now())

        self.temperature = _temperature
        self.temperature_min = min_from_two_values(_temperature, self.temperature_min)
        self.temperature_max = max_from_two_values(_temperature, self.temperature_max)
        self.temperatures = self.temperatures[-20:] + (_temperature, )
        self.temperature_median = statistics.median(self.temperatures)

    def get_temperature(self):
        self.update_temperature_from_influxdb()
        return self.temperature

    def get_temperature_as_string(self):
        self.update_temperature_from_influxdb()
        return "{:.1f}°C".format(self.temperature)

    def is_on(self):
        query = 'SELECT time, level FROM "device.%s-37" ORDER BY DESC LIMIT 1' % (self.ElementID)
        results = tsdb.query(
            query
        )
        _time = results.raw['series'][0]['values'][0][0]
        _level = results.raw['series'][0]['values'][0][1]
        timestamp = "{:%H:%M:%S}".format(datetime.datetime.now())
        # ZAP / VYP
        return True if _level else False

    def is_on_or_off_as_string(self):
        return "ZAP" if self.is_on() else "VYP"

    def check_status_via_url(self):

        try:
            r = requests.post(
                self.url_info_level,
                auth=(self._hub.user, self._hub.password),
                timeout=self._hub.timeout
            )
            r.raise_for_status()
            _level = json.loads(r.text)
        except requests.exceptions.Timeout:
            print("Timeout requesting {}".format(self.url_info_level))
            return "?"
        except requests.exceptions.RequestException as e:
            print("requests exception {} at {}".format(e, datetime.datetime.now()))
            return "?"
        return "ZAP" if _level else "VYP"

    def execute_url_command(self, url):
        try:
            # připravit auth dopředu jako hub.get_auth()
            # podívat se na poziční argument requests.get a případně vynechat jména argumentů
            r = requests.get(url, auth=(self._hub.user, self._hub.password), timeout=self._hub.timeout)
            r.raise_for_status()
        except requests.exceptions.Timeout:
            print("Timeout requesting {}".format(url))
        except requests.exceptions.RequestException as e:
            print("requests exception {} at {}".format(e, datetime.datetime.now()))

    def on(self):
        if DEBUG:
            self.double_check()

        if self.enabled:
            # it's already on
            if DEBUG:
                print("%s it's already on" % self.Name)
                print("to force it use %s" % self.url_command_on)
            return

        self.execute_url_command(self.url_command_on)
        self.enabled = True
        if DEBUG:
            print("%s switched to ON" % self.Name)

    def off(self):
        if DEBUG:
            self.double_check()

        if not self.enabled:
            # it's already off
            if DEBUG:
                print("%s it's already off" % self.Name)
                print("to force it use %s" % self.url_command_off)
            return

        self.execute_url_command(self.url_command_off)
        self.enabled = False
        if DEBUG:
            print("%s switched to OFF" % self.Name)

    def double_check(self):
        _mismatch = False
        if self.enabled and (self.check_status_via_url() == "VYP"):
            print("WARNING: stored as ON, but OFF")
            _mismatch = True
        if not self.enabled and (self.check_status_via_url() == "ZAP"):
            print("WARNING: stored as OFF, but ON")
            _mismatch = True
        if _mismatch:
            self.count_of_mismatches += 1
            print("Count of mismatches", self.count_of_mismatches)
            self.print()
            # sys.exit(1)
        if self.count_of_mismatches > cfg_heating.maximum_of_mismatches:
            self.enabled = not self.enabled
            self.count_of_mismatches = 0

    def print(self):
        print("    Device ID: %2d, Name: %s, %s\n    Element ID: %21s\n"
              % (self._device_id, self.Name, self.Description, self.ElementID))
