#! /usr/bin/env python3
""" Modul pro Fronius střídač """

import requests
import statistics
import datetime

replace_none_with_zero = lambda x: x if x is not None else 0
vybijeni_nebo_nabijeni = lambda x: "nabíjení" if x < 0 else "vybíjení"
prodej_nebo_nakup = lambda x: "prodej" if x < 0 else "nákup"

class Inverter:
    u""" Třída pro práci s Fronius střídačem """

    def __init__(self, host, timeout, rdb):
        self._host = host
        self._timeout = timeout
        self._rdb = rdb
        self.url = "http://%s/solar_api/v1/GetPowerFlowRealtimeData.fcgi" % (
            host
        )
        self.battery_state_of_charge = None
        self.battery_state_of_charge_tuple = tuple()
        self.battery_state_of_charge_median = None
        self.battery_charged_capacity = None
        self.battery_charged_capacity_lowest = 10000
        self.battery_charged_capacity_highest = 0
        self.rel_Autonomy = None
        self.rel_Autonomy_tuple = tuple()
        self.rel_Autonomy_median = None
        self.rel_SelfConsuption = None
        self.rel_SelfConsuption_tuple = tuple()
        self.rel_SelfConsuption_median = None
        # výroba (plus)
        self.P_PV = None
        self.P_PV_tuple = tuple()
        self.P_PV_median = None
        # nejvyšší denní výroba
        self.P_PV_highest = 0
        # spotřeba (mínus)
        self.P_Load = None
        # do (mínus) / z (plus) baterky
        self.P_Akku = None
        self.P_Akku_tuple = tuple()
        self.P_Akku_median = None
        # kupuji ze sítě (plus) / prodávám (mínus)
        self.P_Grid = None
        self.P_Grid_tuple = tuple()
        self.P_Grid_median = None
        # nejvyšší denní prodej (nejnižší hodnota)
        self.P_Grid_highest_sell = 0
        # minimální denní nákup
        # 32767 je max. hodnota v PostgreSQL SMALLINT
        self.P_Grid_lowest_buy = 32767

    def read_data(self):
        u""" Načti powerflow data ze střídače Fronius """
        try:
            r = requests.get(self.url, timeout=self._timeout)
            r.raise_for_status()
            powerflow = r.json()
            self._rdb.save_power_flow_realtime_data(powerflow)
            self.timestamp = powerflow['Head']['Timestamp']
            self.mode = powerflow['Body']['Data']['Site']['Mode']
            self.P_Grid = powerflow['Body']['Data']['Site']['P_Grid']
            if self.P_Grid < self.P_Grid_highest_sell:
                self.P_Grid_highest_sell = self.P_Grid
                self._rdb.save("do_site_max", abs(self.P_Grid))
            if (self.P_Grid > 0) and (self.P_Grid < self.P_Grid_lowest_buy):
                self.P_Grid_lowest_buy = self.P_Grid
                self._rdb.save("ze_site_min", self.P_Grid)
            self.P_Load = powerflow['Body']['Data']['Site']['P_Load']
            x = powerflow['Body']['Data']['Site']['P_Akku']
            # pokud je baterie prázdná, tak místo null nastav nulu
            self.P_Akku = replace_none_with_zero(x)
            self.P_Akku_tuple = self.P_Akku_tuple[-20:] + (self.P_Akku, )
            self.P_Akku_median = statistics.median(self.P_Akku_tuple)
            x = powerflow['Body']['Data']['Site']['P_PV']
            self.P_PV = x if x else 0
            self.P_PV_tuple = self.P_PV_tuple[-20:] + (self.P_PV, )
            self.P_PV_median = statistics.median(self.P_PV_tuple)
            if self.P_PV > self.P_PV_highest:
                self.P_PV_highest = self.P_PV
                self._rdb.save("vyroba_max", self.P_PV)
            self.E_Day = powerflow['Body']['Data']['Site']['E_Day']
            self.E_Year = powerflow['Body']['Data']['Site']['E_Year']
            self.E_Total = powerflow['Body']['Data']['Site']['E_Total']

            x = powerflow['Body']['Data']['Site']['rel_Autonomy']
            self.rel_Autonomy = x if x else 0
            self.rel_Autonomy_tuple = self.rel_Autonomy_tuple[-20:] + (self.rel_Autonomy, )
            self.rel_Autonomy_median = statistics.median(self.rel_Autonomy_tuple)

            x = powerflow['Body']['Data']['Site']['rel_SelfConsumption']
            self.rel_SelfConsuption = x if x else 0
            self.rel_SelfConsuption_tuple = self.rel_SelfConsuption_tuple[-20:] + (self.rel_SelfConsuption, )
            self.rel_SelfConsuption_median = statistics.median(self.rel_SelfConsuption_tuple)

            try:
                self.battery_state_of_charge = powerflow['Body']['Data']['Inverters']['1']['SOC']
            except KeyError as e:
                print("Battery is OFF. Error {} at {}".format(e, datetime.datetime.now()))
                self.battery_state_of_charge = 0
            self.battery_charged_capacity = 72 * self.battery_state_of_charge
            if self.battery_charged_capacity > self.battery_charged_capacity_highest:
                self.battery_charged_capacity_highest = self.battery_charged_capacity
                self._rdb.save("baterie_max", self.battery_charged_capacity)
            if self.battery_charged_capacity < self.battery_charged_capacity_lowest:
                self.battery_charged_capacity_lowest = self.battery_charged_capacity
                self._rdb.save("baterie_min", self.battery_charged_capacity)
            self.battery_state_of_charge_tuple = self.battery_state_of_charge_tuple[-20:] + (self.battery_state_of_charge, )
            self.battery_state_of_charge_median = statistics.median(self.battery_state_of_charge_tuple)
            self.P_Grid_tuple = self.P_Grid_tuple[-100:] + (self.P_Grid, )
            self.P_Grid_median = statistics.median(self.P_Grid_tuple)
        except requests.exceptions.Timeout:
            print("Timeout requesting {}".format(self.url))
        except requests.exceptions.RequestException as e:
            print("requests exception {} at {}".format(e, datetime.datetime.now()))

    def header(self):
        return "nabití   autonomie má sp. výroba spotřeba  síť      baterie {:>6}".format("součet")

    def info(self):
        return "{0:3d}% {1:s} {2:3.0f}% {3:3.0f}% {4:5.0f}W {5:6.0f}W {6:6.0f}W {7:6s} {8:5.0f}W {9:4.0f}W".format(
            self.battery_state_of_charge,
            vybijeni_nebo_nabijeni(self.P_Akku),
            self.rel_Autonomy,
            self.rel_SelfConsuption,
            # výroba (plus)
            self.P_PV,
            # spotřeba (mínus)
            self.P_Load,
            # kupuji ze sítě (plus) / prodávám (mínus)
            self.P_Grid,
            prodej_nebo_nakup(self.P_Grid),
            # do (mínus) / z (plus) baterky
            self.P_Akku,
            self.P_PV + self.P_Load + self.P_Akku + self.P_Grid
        )

    def reset_extrema(self):
        u""" extrama is plural from extrem """
        self.battery_charged_capacity_lowest = 10000
        self.battery_charged_capacity_highest = 0
        # nejvyšší denní výroba
        self.P_PV_highest = 0
        # nejvyšší denní prodej (nejnižší hodnota)
        self.P_Grid_highest_sell = 0
        # minimální denní nákup
        # 32767 je max. hodnota v PostgreSQL SMALLINT
        self.P_Grid_lowest_buy = 32767

    consume_energy = lambda self: self.battery_state_of_charge_median > 70 and self.P_PV_median > 1000 and self.P_Grid_median < -500
    save_energy = lambda self: self.P_Grid_median > -200 and self.battery_state_of_charge_median < 90
