#! /usr/bin/env python3

import re
import os
import sys
import unicodedata
import operator
import time # použit pouze pro sleep()
import datetime
import config
import requests
import json
# moje moduly
import hub as hub_module
import inverter as inverter_module
import room as room_module
import device as device_module
import postgres

__version__ = 1.0
__author__ = 'Jiří Navrátil'
__email__ = 'jiri@navratil.cz'

cfg_number_of_lines_before_next_header = config.number_of_lines_before_next_header
cfg_hub = config.ConfigHub()
cfg_inverter = config.ConfigInverter()
cfg_rdbms = config.ConfigRDBMS()
cfg_heating = config.ConfigHeating()

DEBUG = os.environ.get('DEBUG', False)

def is_time_for_heating():
    _time = datetime.datetime.now().time()
    for i in cfg_heating.intervals_for_heating:
        if i[0] < _time < i[1]:
            return True
    return False

class Dehumidifiers:
    u""" Třída pro práci s odvlhčovačema """

    def __init__(self):
        self.devices = set()

    def add(self, _device):
        self.devices.add(_device)

    def get_set_of_devices(self):
        return self.devices

    def print(self):
        print(self.Name, self.url)
        for i in self.get_set_of_devices():
            i.print()

    def info(self):
        r = ""
        for i in self.get_set_of_devices():
            r += " " + i.check_status_via_url()
        return r

    def on(self):
        for i in self.get_set_of_devices():
            i.on()

    def off(self):
        for i in self.get_set_of_devices():
            i.off()

class Radiatory:
    u""" Třída pro práci s termostaty na radiátorech """

    def __init__(self):
        self.devices = set()

    def add(self, _device):
        self.devices.add(_device)

    def get_set_of_devices(self):
        return self.devices

    def print(self):
        print(self.Name, self.url)
        for i in self.get_set_of_devices():
            i.print()

    def info(self):
        r = ""
        for i in self.get_set_of_devices():
            r += " " + i.check_status_via_url()
        return r

    def on(self):
        for i in self.get_set_of_devices():
            i.on()

    def off(self):
        for i in self.get_set_of_devices():
            i.off()

    def zapnout_kvuli_mrazu(self):
        """ frost protection """
        for i in self.get_set_of_devices():
            if DEBUG:
                print("{}: {:.1f}°C".format(i.Name, i.get_temperature()))
            if i.get_temperature() < cfg_heating.frost_protection:
                elektricky_kotel.on()
                return True
        return False

    def zapnout_nebo_vypnout_kotel(self):
        if not cfg_heating.is_heating_season:
            if DEBUG:
                print("Heating season: NO")
            elektricky_kotel.off()
            return
        if not is_time_for_heating():
            if DEBUG:
                print("Time for heating: NO")
            elektricky_kotel.off()
            return
        zapnout_kotel, vypnout_kotel = None, None
        for i in self.get_set_of_devices():
            if DEBUG:
                print("{}: {:.1f}°C".format(i.Name, i.get_temperature()))
            if i.get_temperature() < cfg_heating.heating_min_temperature:
                zapnout_kotel = i.Description
            elif i.get_temperature() > cfg_heating.heating_max_temperature:
                vypnout_kotel = i.Description
        if zapnout_kotel and vypnout_kotel:
            print("POZOR - požadavek na zapnutí ({}) i na vypnutí ({}) kotle - vypínám kotel".format(zapnout_kotel, vypnout_kotel))
            elektricky_kotel.off()
            return
        if zapnout_kotel:
            elektricky_kotel.on()
        elif vypnout_kotel:
            elektricky_kotel.off()


hub = hub_module.Hub(
    cfg_hub.host, cfg_hub.port,
    cfg_hub.user, cfg_hub.password,
    cfg_hub.timeout
)

none = room_module.Room(hub, 0, "nepřiřazeno")
sklep = room_module.Room(hub, 1, "sklep")
chodba = room_module.Room(hub, 2, "chodba")
kluci = room_module.Room(hub, 3, "pokojíček kluci")
holky = room_module.Room(hub, 4, "pokojíček holčičky")
koupelna = room_module.Room(hub, 5, "koupelna")
pracovna = room_module.Room(hub, 6, "pracovna")
jidelna = room_module.Room(hub, 7, "jídelna")
kuchyn = room_module.Room(hub, 8, "kuchyň")
obyvak = room_module.Room(hub, 9, "obyvák")

jidelna.add_device_to_room(hub)

odvlhcovace = Dehumidifiers()
radiatory = Radiatory()
radiatory_pro_rizeni_kotle = Radiatory()

radiator_v_obyvaku = device_module.Device(
    hub, 35, 0, "ZWayVDev_zway_35-0", "termostat 1", "velký radiátor v obyváku"
)
obyvak.add_device_to_room(radiator_v_obyvaku)
radiatory.add(radiator_v_obyvaku)
radiatory_pro_rizeni_kotle.add(radiator_v_obyvaku)

zasuvka_v_chodbe = device_module.Device(
    hub, 3, 0, "ZWayVDev_zway_3-0", "zásuvka 1", "zásuvka v chodbě"
)
chodba.add_device_to_room(zasuvka_v_chodbe)

elektricky_kotel = device_module.Device(
    hub, 13, 1, "ZWayVDev_zway_13-1", "topení", "ovládání elektrického kotle"
)
sklep.add_device_to_room(elektricky_kotel)

radiator_v_kuchyni = device_module.Device(
    hub, 15, 0, "ZWayVDev_zway_15-0", "termostat 2", "malý radiátor v kuchyni"
)
kuchyn.add_device_to_room(radiator_v_kuchyni)
radiatory.add(radiator_v_kuchyni)
radiatory_pro_rizeni_kotle.add(radiator_v_kuchyni)

odvlhcovac_v_pracovne = device_module.Device(
    hub, 20, 0, "ZWayVDev_zway_20-0", "zásuvka 3", "zásuvka v pracovně s odvhlčovačem"
)
pracovna.add_device_to_room(odvlhcovac_v_pracovne)
odvlhcovace.add(odvlhcovac_v_pracovne)

odvlhcovac_v_koupelne = device_module.Device(
    hub, 21, 0, "ZWayVDev_zway_21-0", "zásuvka 4", "zásuvka v koupelně s odvhlčovačem"
)
koupelna.add_device_to_room(odvlhcovac_v_koupelne)
odvlhcovace.add(odvlhcovac_v_koupelne)

radiator_v_chodbe = device_module.Device(
    hub, 22, 0, "ZWayVDev_zway_22-0", "termostat 3", "velký radiátor v chodbě"
)
chodba.add_device_to_room(radiator_v_chodbe)
radiatory.add(radiator_v_chodbe)

radiator_u_holek = device_module.Device(
    hub, 23, 0, "ZWayVDev_zway_23-0", "termostat 4",
    "velký radiátor v pokojíčku holčiček"
)
holky.add_device_to_room(radiator_u_holek)
radiatory.add(radiator_u_holek)
radiatory_pro_rizeni_kotle.add(radiator_u_holek)

radiator_u_kluku = device_module.Device(
    hub, 24, 0, "ZWayVDev_zway_24-0", "termostat 5",
    "malý radiátor v pokojíčku Pepíčka"
)
kluci.add_device_to_room(radiator_u_kluku)
radiatory.add(radiator_u_kluku)
radiatory_pro_rizeni_kotle.add(radiator_u_kluku)

radiator_v_koupelne = device_module.Device(
    hub, 25, 0, "ZWayVDev_zway_25-0", "termostat 6", "vysoký radiátor v koupelně"
)
koupelna.add_device_to_room(radiator_v_koupelne)
radiatory.add(radiator_v_koupelne)
radiatory_pro_rizeni_kotle.add(radiator_v_koupelne)

odvlhcovac_ve_sklepe = device_module.Device(
    hub, 30, 0, "ZWayVDev_zway_30-0", "zásuvka 2", "zásuvka ve sklepě s odvlhčovačem"
)
sklep.add_device_to_room(odvlhcovac_ve_sklepe)
odvlhcovace.add(odvlhcovac_ve_sklepe)

if cfg_heating.is_heating_season:
    odvlhcovace.on()
else:
    odvlhcovace.off()

rdb = postgres.RDBMS(
    cfg_rdbms.host,
    cfg_rdbms.database,
    cfg_rdbms.user,
    cfg_rdbms.password
)

inverter = inverter_module.Inverter(
    cfg_inverter.host, cfg_inverter.timeout, rdb
)

if __name__ == '__main__':

    today = datetime.date.today()

    wantheader, hdrcnt = False, 1
    while True:
        hdrcnt -= 1
        inverter.read_data()
        if (not hdrcnt or wantheader):
            # vytikni hlavičku (nadpis pro hodnoty)
            print("{} {} {}".format(
                inverter.header(),
                device_module.Device.header(),
                "{:%d.%m.%Y}".format(datetime.date.today())
            ), flush=True)
            elektricky_kotel.double_check()
            wantheader, hdrcnt = False, cfg_number_of_lines_before_next_header
        # vytiskni informační řádek s hodnotami
        print("{} {} {:>13}".format(
            inverter.info(),
            device_module.Device.info(radiator_v_obyvaku, radiator_v_kuchyni, radiator_u_kluku, radiator_u_holek, radiator_v_koupelne, radiator_v_chodbe, elektricky_kotel),
            "{:%H:%M:%S}".format(datetime.datetime.now())
        ), flush=True)
        # akce (pokyny) pro zařízení
        if inverter.consume_energy() or cfg_heating.is_heating_season:
            # odvlhčovače zapni vždy v topné sezóně a mimo ni jen když je dostatek vyráběné energie
            odvlhcovace.on()
        elif inverter.save_energy() and not cfg_heating.is_heating_season:
            # mimo topnou sezónu vypni odvlhčovače, pokud je potřeba šetřit energii
            odvlhcovace.off()
        if not radiatory.zapnout_kvuli_mrazu():
            # v topne sezone, kdyz nehrozi poskozeni mrazem vypni nebo zapni elektrokotel
            radiatory_pro_rizeni_kotle.zapnout_nebo_vypnout_kotel()
        if today != datetime.date.today():
            # kvůli ukládání extrémů zjišťuji, zda je nový den
            today = datetime.date.today()
            # resetuj denní maxima
            inverter.reset_extrema()

        time.sleep(1)
